#ifndef SCENEOBJECTS_HPP_INCLUDED
#define SCENEOBJECTS_HPP_INCLUDED

#include "../../Core/Classes/GameObject.hpp"
#include "../../Core/Classes/DecorativeObject.hpp"
#include "../../Core/Classes/GroundObject.hpp"
#include "../../Core/Classes/WallObject.hpp"
#include "../../Core/Types/Tile.hpp"
#include "../../Core/Types/Convex.hpp"
#include <functional>
#include <cstdint>
#include <vector>
#include <string>

/** @addtogroup Models
* @{ */
class SceneObjects
{
    public:


        typedef enum OBJECT_TYPE
        {
            GAME_OBJECT = (1 << 1),
            DECORATIVE_OBJECT = (1 << 2),
            GROUND_OBJECT =  (1 << 3),
            WALL_OBJECT =  (1 << 4),
            ALL = (GAME_OBJECT | DECORATIVE_OBJECT | GROUND_OBJECT | WALL_OBJECT)
        } OBJECT_TYPE;

        static std::vector<Object> GetAll(OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const Tile& T, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(std::int32_t ID, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const std::string& Name, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const std::vector<std::int32_t>& PossibleIDs, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const std::vector<std::string>& PossibleNames, OBJECT_TYPE ObjectTypes = ALL);

        static std::vector<GameObject> GetAll(const std::function<bool (GameObject&)>& Filter);
        static std::vector<DecorativeObject> GetAll(const std::function<bool (DecorativeObject&)>& Filter);
        static std::vector<GroundObject> GetAll(const std::function<bool (GroundObject&)>& Filter);
        static std::vector<WallObject> GetAll(const std::function<bool (WallObject&)>& Filter);

        static Object Get(const Tile& T, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(std::int32_t ID, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(const std::string& Name, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(const std::vector<std::int32_t>& PossibleIDs, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(const std::vector<std::string>& PossibleNames, OBJECT_TYPE ObjectTypes = ALL);

        static GameObject Get(const std::function<bool (GameObject&)>& Filter);
        static DecorativeObject Get(const std::function<bool (DecorativeObject&)>& Filter);
        static GroundObject Get(const std::function<bool (GroundObject&)>& Filter);
        static WallObject Get(const std::function<bool (WallObject&)>& Filter);

        static Tile GetTileOf(const GameObject& G);
        static Tile GetTileOf(const DecorativeObject& D);
        static Tile GetTileOf(const GroundObject& G);
        static Tile GetTileOf(const WallObject& W);

        static Convex GetConvexOf(const GameObject& G);
        static Convex GetConvexOf(const DecorativeObject& D);
        static Convex GetConvexOf(const GroundObject& G);
        static Convex GetConvexOf(const WallObject& W);
};

/** @} */

#endif // SCENEOBJECTS_HPP_INCLUDED
